@@ -1,35 +1,20 @@
 # -*- coding: utf-8 -*-
-__version__ = '1.1.2'
+__version__ = '2.0.0'
 __versionfull__ = __version__
 
-import base64
 import functools
 import hashlib
 import inspect
 import logging
-import string
 import uuid
 
 from django.conf import settings
 from django.core.cache import cache as default_cache
-
-try:
-    from django.core.cache.backends.base import DEFAULT_TIMEOUT
-except ImportError:  # Django < 1.6 compatibility
-    DEFAULT_TIMEOUT = 300
-
-from ._compat import PY2
+from django.core.cache.backends.base import DEFAULT_TIMEOUT
+from django.utils.encoding import force_bytes
 
 logger = logging.getLogger(__name__)
 
-# Used to remove control characters and whitespace from cache keys.
-valid_chars = set(string.ascii_letters + string.digits + '_.')
-delchars = ''.join(c for c in map(chr, range(256)) if c not in valid_chars)
-if PY2:
-    null_control = (None, delchars)
-else:
-    null_control = (dict((k, None) for k in delchars),)
-
 
 def function_namespace(f, args=None):
     """
@@ -71,11 +56,9 @@ def function_namespace(f, args=None):
             name = f.__name__
 
     ns = '.'.join((module, name))
-    ns = ns.translate(*null_control)
 
     if instance_token:
         ins = '.'.join((module, name, instance_token))
-        ins = ins.translate(*null_control)
     else:
         ins = None
 
@@ -134,7 +117,7 @@ class Memoizer(object):
         return funcname + '_memver'
 
     def _memoize_make_version_hash(self):
-        return base64.b64encode(uuid.uuid4().bytes)[:6].decode('utf-8')
+        return uuid.uuid4().hex
 
     def _memoize_version(self, f, args=None,
                          reset=False, delete=False, timeout=DEFAULT_TIMEOUT):
@@ -197,21 +180,15 @@ class Memoizer(object):
                 altfname = fname
 
             if callable(f):
-                keyargs, keykwargs = self._memoize_kwargs_to_args(f,
-                                                                 *args,
-                                                                 **kwargs)
+                keyargs, keykwargs = self._memoize_kwargs_to_args(
+                    f, *args, **kwargs
+                )
             else:
                 keyargs, keykwargs = args, kwargs
 
-            try:
-                updated = "{0}{1}{2}".format(altfname, keyargs, keykwargs)
-            except AttributeError:
-                updated = "%s%s%s" % (altfname, keyargs, keykwargs)
-
-            cache_key = hashlib.md5()
-            cache_key.update(updated.encode('utf-8'))
-            cache_key = base64.b64encode(cache_key.digest())[:16]
-            cache_key = cache_key.decode('utf-8')
+            cache_key = hashlib.md5(
+                force_bytes((altfname, keyargs, keykwargs))
+            ).hexdigest()
             cache_key += version_data
 
             if self.cache_prefix:
@@ -239,7 +216,7 @@ class Memoizer(object):
                 arg = repr(args[0])
                 arg_num += 1
             elif argspec.args[i] in kwargs:
-                arg = kwargs[argspec.args[i]]
+                arg = kwargs.pop(argspec.args[i])
             elif arg_num < len(args):
                 arg = args[arg_num]
                 arg_num += 1
@@ -271,7 +248,12 @@ class Memoizer(object):
 
             new_args.append(arg)
 
-        return tuple(new_args), {}
+        # If there are any missing varargs then
+        # just append them since consistency of the key trumps order.
+        if argspec.varargs and args_len < len(args):
+            new_args.extend(args[args_len:])
+
+        return tuple(new_args), kwargs
 
     def memoize(self, timeout=DEFAULT_TIMEOUT, make_name=None, unless=None):
         """
@@ -305,8 +287,9 @@ class Memoizer(object):
                     The original undecorated function. readable only
 
                 **cache_timeout**
-                    The cache timeout value for this function. For a custom value
-                    to take affect, this must be set before the function is called.
+                    The cache timeout value for this function. For a custom
+                    value to take affect, this must be set before the function
+                    is called.
 
                     readable and writable
 
@@ -320,8 +303,8 @@ class Memoizer(object):
                         for that amount of time. Unit of time is in seconds.
         :param make_name: Default None. If set this is a function that accepts
                           a single argument, the function name, and returns a
-                          new string to be used as the function name. If not set
-                          then the function name is used.
+                          new string to be used as the function name.
+                          If not set then the function name is used.
         :param unless: Default None. Cache will *always* execute the caching
                        facilities unelss this callable is true.
                        This will bypass the caching entirely.
@@ -336,22 +319,31 @@ class Memoizer(object):
                     return f(*args, **kwargs)
 
                 try:
-                    cache_key = decorated_function.make_cache_key(f, *args, **kwargs)
+                    cache_key = decorated_function.make_cache_key(
+                        f, *args, **kwargs
+                    )
                     rv = self.get(cache_key)
                 except Exception:
                     if settings.DEBUG:
                         raise
-                    logger.exception("Exception possibly due to cache backend.")
+                    logger.exception(
+                        "Exception possibly due to cache backend."
+                    )
                     return f(*args, **kwargs)
 
                 if rv is None:
                     rv = f(*args, **kwargs)
                     try:
-                        self.set(cache_key, rv, timeout=decorated_function.cache_timeout)
+                        self.set(
+                            cache_key, rv,
+                            timeout=decorated_function.cache_timeout
+                        )
                     except Exception:
                         if settings.DEBUG:
                             raise
-                        logger.exception("Exception possibly due to cache backend.")
+                        logger.exception(
+                            "Exception possibly due to cache backend."
+                        )
                 return rv
 
             decorated_function.uncached = f
@@ -359,7 +351,9 @@ class Memoizer(object):
             decorated_function.make_cache_key = self._memoize_make_cache_key(
                 make_name, decorated_function
             )
-            decorated_function.delete_memoized = lambda: self.delete_memoized(f)
+            decorated_function.delete_memoized = (
+                lambda: self.delete_memoized(f)
+            )
 
             return decorated_function
         return memoize
@@ -367,8 +361,9 @@ class Memoizer(object):
     def delete_memoized(self, f, *args, **kwargs):
         """
         Deletes the specified functions caches, based by given parameters.
-        If parameters are given, only the functions that were memoized with them
-        will be erased. Otherwise all versions of the caches will be forgotten.
+        If parameters are given, only the functions that were memoized with
+        them will be erased. Otherwise all versions of the caches will be
+        forgotten.
 
         Example::
 
@@ -436,16 +431,20 @@ class Memoizer(object):
             >>> adder2.add(3)
             3.72341788
 
-        :param fname: Name of the memoized function, or a reference to the function.
-        :param \*args: A list of positional parameters used with memoized function.
-        :param \**kwargs: A dict of named parameters used with memoized function.
+        :param fname: Name of the memoized function, or a reference to the
+                      function.
+        :param \*args: A list of positional parameters used with memoized
+                       function.
+        :param \**kwargs: A dict of named parameters used with memoized
+                          function.
 
         .. note::
 
-            django-memoize uses inspect to order kwargs into positional args when
-            the function is memoized. If you pass a function reference into ``fname``
-            instead of the function name, django-memoize will be able to place
-            the args/kwargs in the proper order, and delete the positional cache.
+            django-memoize uses inspect to order kwargs into positional args
+            when the function is memoized. If you pass a function reference
+            into ``fname`` instead of the function name, django-memoize will
+            be able to place the args/kwargs in the proper order, and delete
+            the positional cache.
 
             However, if ``delete_memoized`` is just called with the name of the
             function, be sure to pass in potential arguments in the same order
@@ -454,20 +453,24 @@ class Memoizer(object):
 
         .. note::
 
-            django-memoize maintains an internal random version hash for the function.
-            Using delete_memoized will only swap out the version hash, causing
-            the memoize function to recompute results and put them into another key.
+            django-memoize maintains an internal random version hash for the
+            function. Using delete_memoized will only swap out the version
+            hash, causing the memoize function to recompute results and put
+            them into another key.
 
-            This leaves any computed caches for this memoized function within the
-            caching backend.
+            This leaves any computed caches for this memoized function within
+            the caching backend.
 
             It is recommended to use a very high timeout with memoize if using
-            this function, so that when the version has is swapped, the old cached
-            results would eventually be reclaimed by the caching backend.
+            this function, so that when the version has is swapped, the old
+            cached results would eventually be reclaimed by the caching
+            backend.
         """
         if not callable(f):
-            raise DeprecationWarning("Deleting messages by relative name is no longer"
-                          " reliable, please switch to a function reference")
+            raise DeprecationWarning(
+                "Deleting messages by relative name is no longer"
+                " reliable, please switch to a function reference"
+            )
 
         try:
             if not args and not kwargs:
@@ -493,8 +496,10 @@ class Memoizer(object):
             in the cache backend.
         """
         if not callable(f):
-            raise DeprecationWarning("Deleting messages by relative name is no longer"
-                          " reliable, please use a function reference")
+            raise DeprecationWarning(
+                "Deleting messages by relative name is no longer"
+                " reliable, please use a function reference"
+            )
 
         try:
             self._memoize_version(f, delete=True)
